##############################################################################
# Data loading into the application is contained within this package
# 

package GShowTV::Data::Loader;

use strict;
use warnings;

use Storable qw/fd_retrieve/;
use LWP::Simple qw/get/;

use Data::Dumper;

use Glib qw/TRUE FALSE G_PRIORITY_HIGH_IDLE G_PRIORITY_DEFAULT_IDLE/;
use Gtk2::Helper;
use GShowTV::Constants;
use GShowTV::Debug qw/debug/;
use GShowTV::Strings qw/is_old_date myglob/;
use GShowTV::UI::ProgressWindow;
use GShowTV::Data::IO;
use Locale::TextDomain qw/gshowtv/;

use Glib::Object::Subclass
    Glib::Object::, # Parent class
	signals => {
		xmltv_data_ready => {
			method      => 'default_signal_handler',
			flags       => [qw/run-first/],
			return_type => undef, # void return
			param_types => [qw/Glib::Scalar/], # instance and data are automatic
		},
		images_ready => {
			method      => 'default_signal_handler',
			flags       => [qw/run-first/],
			return_type => undef, # void return
			param_types => [qw/Glib::Scalar/], # instance and data are automatic
		},
		error => {
			method      => 'default_signal_handler',
			flags       => [qw/run-first/],
			return_type => undef, # void return
			param_types => [qw/Glib::Scalar/], # instance and data are automatic
		},
		info => {
			method      => 'default_signal_handler',
			flags       => [qw/run-first/],
			return_type => undef, # void return
			param_types => [qw/Glib::Scalar/], # instance and data are automatic
		},
	};

sub INIT_INSTANCE {
    my $self = shift;
	$self->{progress} = GShowTV::UI::ProgressWindow->new();
	$self;
}

sub set_config {
	my $self = shift;
	my $oldconfig = $self->{config};
	my $config = shift;
	$self->{config} = $config;
	
	debug("Called");
	
	if (!defined ($oldconfig)) {
		return;
	}

	if ($config->{xmltvfile} ne $oldconfig->{xmltvfile}) {
		$self->load_xmltv_data();
		return;
	}
	if ($config->{loadimages} && $config->{loadimages} ne $oldconfig->{loadimages}) {
		$self->load_images();
	}

	if (defined ($self->{main_loop_id})) { # We are running
		my $sid = Glib::Timeout->add(1, sub { my $self = shift; $self->main_loop_iteration(); FALSE; }, $self);
	}
}

sub start {
	my $self = shift;
	debug("Starting xmltv data loader");

	my $sid = Glib::Timeout->add(1 * 60 * 1000, \&main_loop_iteration, $self);
	$self->{main_loop_id} = $sid;

	# Run the main loop once when starting. (Otherwise we'd wait for a whole minute before having any data available.)
	Glib::Timeout->add(1, sub {my $self = shift; $self->main_loop_iteration(); FALSE; }, $self);
}

sub main_loop_iteration {
	my $self = shift;

	return TRUE unless defined ($self->{config}->{xmltvfile});
	
	my ($xmltv_since, $xmltv_size) = (stat $self->{config}->{xmltvfile})[9,7];
	my $channels_since = (stat $GShowTV::Constants::confdir . "/channels.dat")[9];

	if (!defined ($channels_since) || $channels_since <= $xmltv_since) {
		{ no warnings;
		  debug("Channels file is older than the xmltv file. ($channels_since < $xmltv_since)");
		}
		if ($xmltv_size != 0) {
			debug ("Start parsing xmltv data");
			$self->parse_xmltv_data();
		} else {
			debug("XMLTV file is empty, not parsing.");
			if (! defined ($self->{once_loaded})) {
				$self->load_xmltv_data();
			}
		}
	} elsif (! defined ($self->{once_loaded})) {
		$self->load_xmltv_data();
	} elsif (defined ($self->{once_loaded}) && $self->{old_channels_since} < $channels_since) {
		debug("Detected preparsed xmltv data");
		$self->load_xmltv_data();
	}

	$self->{old_channels_since} = $channels_since;
	
	TRUE;
}

sub parse_xmltv_data {
	my $self = shift;
	
	if (defined $self->{parse_pipe}) {
		debug("Parsing is already going on, refusing to parse.");
		return;
	}

	# Use the special variable $0 to run the gshowtv, then we can guarantee that it's
	# this particular version of gshowtv that we are running.

	open $self->{parse_pipe}, '-|', $GShowTV::Constants::run_command . 
		" --parse --slave " . ($GShowTV::Debug::DEBUG ? "-d" : "") . " " . $self->{config}->{xmltvfile}
		or die "Failed running gshowtv parser subprocess\n";

	my $tag;
	$tag = Gtk2::Helper->add_watch(fileno($self->{parse_pipe}), 'in', sub {
		my ($f, $d, $self) = @_;
		
		if (! defined ($self->{parse_pipe})) {
			debug("Parse pipe is not available");
			Gtk2::Helper->remove_watch($tag);
		} elsif ( eof( $self->{parse_pipe} ) ) {
			close( $self->{parse_pipe} );
			$self->{parse_pipe} = undef;
			Gtk2::Helper->remove_watch($tag);

			if ($self->{progress}->visible()) {
				$self->{progress}->hide();
				debug("watcher exiting..");
				$self->load_xmltv_data();
			} elsif (! defined ($self->{program_tree})) {
				debug("No data has yet been loaded, therefore load it in now");
				$self->load_xmltv_data();
			} else {
				debug("progress wasnt visible, hence there were no programs to load");
			}
		} else {
			my $pipe = $self->{parse_pipe};
			my $line = <$pipe>;
			$line =~ m/^(\d+)\/(\d+)/;
			my $counter = $1;
			my $max     = $2;

			# Show the dialog here if there really are something to load
									
			if ($counter == 1 && $max == 1) {
				$self->signal_emit("error", __"Error: XMLTV file doesn't contain any data.");
			} 
			elsif ($counter == 1 && $max > 1) {
				debug("Expecting programmes $max, hence showing progress window.");
				
				$self->{progress}->set_label(__"Parsing XMLTV data");
				$self->{progress}->set_fraction(0);
				$self->{progress}->set_text("0%");
				$self->{progress}->show_all();
			}
			if ($counter % 100 == 0) {
				$self->{progress}->set_fraction($counter / $max);
				$self->{progress}->set_text(int ($counter / $max * 100) . "%");
			}

			if ($line =~ /\*$/) {
				debug("First day is now parsed, load it in.");
				$self->load_xmltv_data(TRUE);
			}
		}
		TRUE;
	}, $self);
}

sub load_xmltv_data {
	my $self = shift;
	my $disable_dialog = shift; # Set when synchronously parsing and loading xmltv data.
	
	# Main utility to load the xmltv data. (Parsed, eval-able)
	debug("Loading xmltv data");

	$self->load_channels();

	if (! $disable_dialog) {
		$self->{progress}->set_label(__"Reading XMLTV data");
		$self->{progress}->set_fraction(0);
		$self->{progress}->set_text("0%");
		$self->{progress}->show_all();
	}

	# Initialized all individual data files to the task array
	my @task = ();

	# Clear old dates
	my @dates = ();

	# Clear old programs
	$self->{program_tree} = undef;
	
	foreach my $date (glob $GShowTV::Constants::datadir . "/*") {
		if ($date =~ /.*\/(\d{8})$/) {
			my $d = $1;

			# TODO Decide what to do with old dates. They might be interesting to keep.

			next if (is_old_date($d));

			push @dates, $d;
			debug("Going to load $d");

			foreach my $chan (glob $date . "/*") {
				push @task, $chan;
			}
		} else {
			debug("unknown file $date");
		}
	}

	my $task_max   = scalar @task;

	Glib::Idle->add(sub {
		my $task = shift;
		
		if (scalar @$task == 0) {
			$self->{once_loaded} = TRUE;
			
			if (! $disable_dialog) {
				$self->{progress}->hide_all();
			}
			$self->signal_emit("xmltv_data_ready", $self->{program_tree});
			$self->load_images($disable_dialog);
			
			$self->{first_date} = undef;
			$self->{first_date_emitted} = undef;

			my $showcount = 0;

			my ($c, $d);
			foreach $d (values %{$self->{program_tree}}) {
				foreach $c (values %$d) {
					$showcount = $showcount + scalar values %$c;
				}
			}

  			$self->signal_emit("info", sprintf(__"Currently in XMLTV db: %s tv shows, %s channels.", $showcount, scalar keys %{$self->{channel_tree}}));

			FALSE;
		} else {
			my $file = shift @$task;

			my $buf = "";

			my $progs1;

			if (open READ, "<:utf8", $file) {
				$progs1 = fd_retrieve *READ;
				close READ;
			}

			my $date = undef;
			
			foreach my $p (@$progs1) {
				my $channel = $p->{channel};
				my $pid     = $p->{id};
				$date    = $p->{start};
				$date =~ s/(\d{8}).*/$1/;

				$self->{program_tree}->{$date}->{$channel}->{$pid} = $p;
			}

			if (! defined ($self->{first_date})) {
				$self->{first_date} = $date;
			}

			if (! defined ($self->{once_loaded}) && $self->{first_date} ne $date && ! defined ($self->{first_date_emitted})) {
				$self->{first_date_emitted} = TRUE;
				debug("Signalling first date");
				$self->signal_emit("xmltv_data_ready", $self->{program_tree});
#				$self->{program_tree} = undef; # Loose the current tree, it's been sent
			}
			
			if (scalar @$task % 10 == 0 && !$disable_dialog) {
				$self->{progress}->set_fraction(($task_max - scalar @$task) / $task_max);
				$self->{progress}->set_text(int (($task_max - scalar @$task) / $task_max * 100) . "%");
			}
			1;
		}
	}, \@task, $self); # Cannot set priority, have to hope for LIFO
}

sub load_images {
	my $self = shift;
	my $disable_dialog = shift;

	return if (!defined $self->{config}->{loadimages} || ! $self->{config}->{loadimages});

	# Initialized all individual data files to the task array
	my @task = ();

	foreach my $c (values %{$self->{channel_tree}}) {
		next unless defined ($c->{icon_url});

		my $glob = myglob($GShowTV::Constants::imagedir . "/" . $c->{id} . ".*");
		next if (defined ($glob) && -e $glob);

		push @task, $c;
	}

	debug("Need to load " . scalar @task . " images");

	return if (scalar @task == 0);

	my $task_max   = scalar @task;

	if (! $disable_dialog) {
		$self->{progress}->set_label(__"Loading images");
		$self->{progress}->set_fraction(0);
		$self->{progress}->set_text("0%");
		$self->{progress}->show_all();
	}

	$self->{anImageIsLoaded} = FALSE;
	
	Glib::Idle->add(sub {
		my $task = shift;

		if (scalar @$task == 0) {
			if ($self->{anImageIsLoaded}) {
				$self->signal_emit("images_ready", $self->{program_tree});
			}
			delete $self->{anImageIsLoaded};
			if (! $disable_dialog) {
				$self->{progress}->hide_all();
			}
			FALSE;
		} else {
			my $c = shift @$task;

			debug("Loading $c->{icon_url}");

			my $content = get($c->{icon_url});
			if (defined ($content)) {
				$self->save_image($c, $content);
			} else {
				debug("Can't fetch icon $c->{icon_url}");
			}
			if (! $disable_dialog) {
				$self->{progress}->set_fraction(($task_max - scalar @$task) / $task_max);
				$self->{progress}->set_text(int (($task_max - scalar @$task) / $task_max * 100) . "%");
			}
			TRUE;
		}
	}, \@task);
}

sub load_channels {
	my $self = shift;

	$self->{channel_tree} = GShowTV::Data::IO::read_channels();
}

sub get_channels {
	my $self = shift;
	$self->{channel_tree};
}

sub get_programs {
	my $self = shift;
	$self->{program_tree};
}

sub save_image {
	my $self = shift;
	my $c    = shift;
	my $data = shift;
	 
	$c->{icon_url} =~ /.*(\.[^\.]+)/;

	if (open (WRITE, ">" . $GShowTV::Constants::imagedir . "/" . $c->{id} . $1)) {
		print WRITE $data;
		close WRITE;

		$self->{anImageIsLoaded} = TRUE;

		# Convert the loaded image to png		
		my $pixbuf = Gtk2::Gdk::Pixbuf->new_from_file($GShowTV::Constants::imagedir . "/" . $c->{id} . $1);
		$pixbuf->save($GShowTV::Constants::imagedir . "/" . $c->{id} . ".png", "png");
		unlink ($GShowTV::Constants::imagedir . "/" . $c->{id} . $1);
	} else {
		debug("Can't write image $GShowTV::Constants::imagedir/$c->{id}.$1");
	}
}

sub default_signal_handler {}

1;
