##############################################################################
# PVR Integration module
# 

package GShowTV::Data::PVR;

use strict;
use warnings;

use Data::Dumper;

use Glib qw/TRUE FALSE/;
use Gtk2::Helper;
use GShowTV::Constants;
use GShowTV::Debug qw/debug/;
use GShowTV::Strings qw/is_old_date myglob/;
use Locale::TextDomain qw/gshowtv/;

use Glib::Object::Subclass
    Glib::Object::, # Parent class
	signals => {
		pvr_data_ready => {
			method      => 'default_signal_handler',
			flags       => [qw/run-first/],
			return_type => undef, # void return
			param_types => [qw/Glib::Scalar/], # instance and data are automatic
		},
		pvr_channels_ready => {
			method      => 'default_signal_handler',
			flags       => [qw/run-first/],
			return_type => undef, # void return
			param_types => [qw/Glib::Scalar/], # instance and data are automatic
		},
		info => {
			method      => 'default_signal_handler',
			flags       => [qw/run-first/],
			return_type => undef, # void return
			param_types => [qw/Glib::Scalar/], # instance and data are automatic
		},
		error => {
			method      => 'default_signal_handler',
			flags       => [qw/run-first/],
			return_type => undef, # void return
			param_types => [qw/Glib::Scalar/], # instance and data are automatic
		},
	};

sub INIT_INSTANCE {
    my $self = shift;
	$self;
}

sub set_config {
	my $self = shift;
	my $oldconfig = $self->{config};
	my $config = shift;
	$self->{config} = $config;
	
	debug("Called");
	
	if (!defined ($oldconfig)) {
		return;
	}

	if (!defined ($config->{pvrinterface})) {
		return;
	}

	if ($config->{pvrinterface} ne $oldconfig->{pvrinterface}) {
		$self->load_data();
		return;
	}

	if (defined ($self->{main_loop_id})) { # We are running
		my $sid = Glib::Timeout->add(1, sub { my $self = shift; $self->main_loop_iteration(); FALSE; }, $self);
	}
}

sub start {
	my $self = shift;
	debug("Starting xmltv data loader");

	my $sid = Glib::Timeout->add(5 * 60 * 1000, \&main_loop_iteration, $self);
	$self->{main_loop_id} = $sid;

	# Run the main loop once when starting. (Otherwise we'd wait for a whole minute before having any data available.)
	Glib::Timeout->add(1, sub { my $self = shift; $self->main_loop_iteration(); FALSE; }, $self);
}

sub main_loop_iteration {
	my $self = shift;

	return TRUE unless defined ($self->{config}->{pvrinterface});

	debug("listing programs from PVR");

	my $pipe;

	if (open $pipe, '-|', $self->{config}->{pvrinterface} . " list") {
		binmode($pipe, ":utf8");
		my $buffer = "";
		my $tag;

		$tag = Gtk2::Helper->add_watch(fileno($pipe), 'in', sub {
			my ($fno, $in, $self) = @_;
			if ( eof( $pipe ) ) {
				close( $pipe );
				$pipe = undef;
				Gtk2::Helper->remove_watch($tag);
				$self->parse_pvr_list($buffer);
			} else {
				my $line = <$pipe>;
				$buffer = $buffer . $line;
			}
			TRUE;
		}, $self);
	} else {
		debug("cannot run pvr " . $self->{config}->{pvrinterface} . ": $!");
	}
	TRUE;
}

sub parse_pvr_list {
	my $self = shift;
	my $buffer = shift;
	my $p;
	my $pvr_program_tree;

	foreach $_ (split "\n", $buffer) {
		if (/^title\s*=\s(.*)/) {
			if (defined ($p)) {
				if (defined ($p->{channel}) && defined ($p->{start})) {
					$p->{id} = $p->{channel} . "." . $p->{start};
					$pvr_program_tree->{$p->{id}} = $p;
				} else {
					debug("skipping entry " . $p->{title} . " since data is incomplete.");
				}
				$p = undef;
			}
			$p->{title} = $1;
		} elsif (/^desc\s*=\s(.*)/) {
			$p->{desc} = $1;
		} elsif (/^channel\s*=\s(.*)/) {
			$p->{channel} = $1;
		} elsif (/^start\s*=\s(.*)/) {
			$p->{start} = $1;
		} elsif (/^stop\s*=\s(.*)/) {
			$p->{stop}  = $1;
		} elsif (/^category\s*=\s(.*)/) {
			$p->{category} = $1;
		} elsif (/^file\s*=\s(.*)/) {
			$p->{file} = $1;
		}
	}

	if (defined ($p)) {
		if (defined ($p->{channel}) && defined ($p->{start})) {
			$p->{id} = $p->{channel} . "." . $p->{start};
			$pvr_program_tree->{$p->{id}} = $p;
		} else {
			debug("skipping entry " . $p->{title} . " since data is incomplete.");
		}
	}

	# We have to go through all the channels here, and try to match the pvr channel id
	# to a xmltv channel id. NOT IMPLEMENTED
  
	# Update the channel tree with channels that were given by the PVR interface
	# HOWEVER: By default make them non visible.

	foreach my $p (values %$pvr_program_tree) {
		unless (defined ($self->{pvr_channel_tree}->{$p->{channel}})) {
			my $c = { id      => $p->{channel},
					  pvrid   => $p->{channel},
					  name    => $p->{channel},
					  order   => 1001,
					  visible => 0 };
					  
			$self->{pvr_channel_tree}->{$c->{id}} = $c;
		}
	}

	debug("Loaded " . scalar keys (%$pvr_program_tree) . " programs.");

	$self->signal_emit("pvr_data_ready", $pvr_program_tree);
}

sub get_channels {
	my $self = shift;
	$self->{pvr_channel_tree};
}

sub get_pvr_info {
	my $self = shift;
	debug("Showing pvr info");

	return undef unless $self->test_arguments(undef, undef, "info");

	if (open READ, '-|', $self->{config}->{pvrinterface} . " info") {
		binmode(READ, ":utf8");
		my $buffer = "";

		while (<READ>) {
			chomp();
			$buffer = $buffer . $_;
		}
		close READ;

		$buffer =~ s/<br>/\n/gi;
		$buffer;
	} else {
		debug("Cannot get info from pvr " . $self->{config}->{pvrinterface});
		undef;
	}
}

sub get_pvr_interface_version {
	my $self = shift;

	return undef unless $self->test_arguments(undef, undef, "interfaceversion");

	my $v = `$self->{config}->{pvrinterface} interfaceversion`;

	$v =~ s/\n.*//gm;

	debug($self->{config}->{pvrinterface} . " version is $v");

	$v;
}

sub record {
	my $self = shift;
	my $p    = shift;
	my $c	 = shift;

	return undef unless $self->test_arguments($p, $c, "record");

	my $stop = $p->{'stop'}   if (defined ($p->{'stop'}));
	my $start = $p->{'start'} if (defined ($p->{'start'}));

	if (! defined ($start)) {
		$self->signal_emit("info", __"Cannot set recording, start time is not known.");
		return undef;
	}
	
	if (! defined ($stop)) {
		$self->signal_emit("info", __"Cannot set recording, stop time is not known.");
		return undef;
	}

    if (is_old_date($stop)) {
		$self->signal_emit("info", __"Cannot set recording, this show has already stopped.");
		return undef;
    }

	my $status = system ($self->{config}->{pvrinterface},
		("record",
			$c->{pvrid},
			$start,
			$stop,
			$p->{'title'},
			$p->{desc}
		));

	if ($status == 1) {
		$self->signal_emit("error", __"The recording cannot be initialized, due to overlapping tv shows.");
		return undef;
	} elsif ($status > 1) {
		$self->signal_emit("error", __"The recording cannot be initialized, due to an unknown error.");
		return undef;
	}

	return TRUE;
}

sub cancel {
	my $self = shift;
	my $p    = shift;
	my $c    = shift;

	return undef unless $self->test_arguments($p, $c, "cancel");

	my $status = system($self->{config}->{pvrinterface}, 
						("cancel", $c->{pvrid}, $p->{start}));

	if ($status != 0) {
		$self->signal_emit("error", __"Cannot cancel recording due to an unknown error");
		return undef;
	}
	return TRUE;
}

sub filemanager {
	my $self = shift;
	
	return undef unless $self->test_arguments(undef, undef, "filemanager");
	
	system($self->{config}->{pvrinterface} . " filemanager &");
	return TRUE;
}

sub delete {
	my $self = shift;
	my $p    = shift;
	my $c    = shift;

	return undef unless $self->test_arguments($p, $c, "delete");

	my $status = system($self->{config}->{pvrinterface}, 
  					("delete", $c->{pvrid}, $p->{start}, $p->{file}));

	if ($status != 0) {
		$self->signal_emit("error", __"Couldn't delete file due to an unknown error");
		return undef;
	}
	return TRUE;
}

sub watch {
	my $self = shift;
	my $p    = shift;
	my $c    = shift;

	return undef unless $self->test_arguments($p, $c, "watch");
	
	my $status = system($self->{config}->{pvrinterface}, 
  					("watch", $c->{pvrid}, $p->{start}, $p->{file}));

	if ($status == 1) {
		$self->signal_emit("error", sprintf (__"File '%s' does not exist.", $p->{'file'}));
		return undef;
	} elsif ($status != 0) {
		$self->signal_emit("error", sprintf (__"Failed to open file %s.", $p->{'file'}));
		return undef;
	}
	return TRUE;
}

sub test_arguments {
	my $self = shift;
	my $p    = shift;
	my $c    = shift;
	my $op   = shift;

	debug("Checking required arguments for operation $op");

	if (!defined ($self->{config}) || ! defined ($self->{config}->{pvrinterface}) || $self->{config}->{pvrinterface} eq "") {
		$self->signal_emit("info", __"No PVR interface configured");
		return FALSE;
	}

	if ($op eq "watch" || $op eq "record" || $op eq "cancel" || $op eq "delete") {
		unless (defined ($p)) {
			$self->signal_emit("info", __"No program selected!");
			return FALSE;
		}

		unless (defined ($c)) {
			$self->signal_emit("info", __"No program selected!"); # Same signal, since this should be always set when program is set
			return FALSE;
		}
		
		unless (defined $c->{pvrid}) {
		
			if ($op eq "record") {
				$self->signal_emit("info", __"Cannot set recording, no PVR channel name configured.");
			} elsif ($op eq "delete") {
				$self->signal_emit("info", __"Cannot delete a recording, no PVR channel name configured.");
			} elsif ($op eq "cancel") {
				$self->signal_emit("info", __"Cannot cancel a recording, no PVR channel name configured.");
			} elsif ($op eq "watch") {
				$self->signal_emit("info", __"Cannot watch a recording, no PVR channel name configured.");
			}
			return FALSE;
		}
	}
	return TRUE;
}

1;
