###############################################################################
# This package is responsible for parsing the xmltv data file into perl data
# structure that is synchronized to disk with Storable. 
# (Start up speed _slightly_ faster than when alwasy parsing the XML.
###############################################################################

package GShowTV::Data::Parser;

use strict;
use warnings;
use utf8;
use Encode qw/decode/;
use XMLTV;
use Storable qw/store_fd fd_retrieve/;
use GShowTV::Debug qw/debug/;

# Data placeholders

my $channels;          # id -> channel
my $tree     = undef;  # date -> channel -> program
my $encoding = undef;
my $credits  = undef;
my $max      = 0;
my $counter  = 1;
my $slave    = 0;
my $first_date = undef;
my $first_date_emitted = undef;
my $channels_persisted = undef;
my $programs_persisted = undef;

my $confdir;
my $datadir;

sub _encoding_cb( $ ) { $encoding = shift }
sub _credits_cb ( $ ) { $credits  = shift }

sub _decodestr {
  if ( $encoding ) { return decode($encoding, $_[0]) } else { return $_[0] };
};

sub _channel_cb(  $ ) {
  my $c    = shift;
  my $d    = _decodestr($c->{'display-name'}->[0]->[0]);
  my $chan = {
	      id       => $c->{id},
	      name     => $d,
	      order    => 1000,   # By default new channels are put to last place
	      visible  => 1       # By default all new channels are visible
	     };

	if (defined ($c->{icon}->[0]->{src})) {
		$chan->{icon_url} = $c->{icon}->[0]->{src},
	}
	
  $channels->{$c->{id}} = $chan;
  $channels_persisted = undef;
}

sub _programme_cb( $ ) {
  my $p = shift;
  $programs_persisted = undef;
    
  persist_channels() unless defined ($channels_persisted);

  my $title    = undef;
  my $channel  = undef;
  my $start    = undef;
  my $stop     = undef;
  my $desc     = undef;
  my $cat      = undef;
  my $subtitle = undef;

  {
    no warnings;

    $title    = _decodestr($p->{'title'}->[0]->[0]);
    $channel  = _decodestr($p->{'channel'});
    $start    = defined ($p->{'start'})     ? _decodestr(substr($p->{'start'}, 0, 14)) : undef;
    $stop     = defined ($p->{'stop'})      ? _decodestr(substr($p->{'stop'}, 0, 14))  : undef;
    $desc     = defined ($p->{desc})        ? _decodestr($p->{'desc'}->[0]->[0])       : undef;
    $cat      = defined ($p->{category})    ? _decodestr($p->{'category'}->[0]->[0])   : undef;
    $subtitle = defined ($p->{'sub-title'}) ? _decodestr($p->{'sub-title'}->[0]->[0])  : undef;
  }

  my $id = $channel . "." . $start;

  my $pp = {
	    id       => $id,
	    title    => $title,
	    start    => $start,
	    stop     => $stop,
	    channel  => $channel
	   };

  $pp->{subtitle} = $subtitle if defined ($subtitle);
  $pp->{desc}     = $desc     if defined ($desc);
  $pp->{category} = $cat      if defined ($cat);

  my $date = substr( $start, 0, 8 );

  push @{$tree->{$date}->{$channel}}, $pp;
  
  if (!defined ($first_date)) {
  	$first_date = $date;
  }
  
  if ($first_date ne $date && ! defined ($first_date_emitted)) {
	$first_date_emitted = 1;
	persist_programs() unless defined ($programs_persisted);
	print $counter . "/" . $max . "*\n" if ($slave);
  } else {
	print $counter . "/" . $max . "\n" if ($slave);
  }

  $counter++;
}

sub parse {
	$confdir = shift;
	$datadir = shift;
	$slave   = shift;
	my @files   = @_;

	$| = 1;

	my $all_start = time;

	foreach my $file (@files) {
    	my $document = "";
    	my $start    = time;

    	$channels = undef;
    	$tree     = undef;
    	$encoding = undef;
    	$credits  = undef;
    	$max      = 0;
    	$counter  = 1;

    	if (defined ($file) && open (READ, $file)) {
			while (<READ>) {
				$document = $document . $_;
			}
			close READ;
    	} else {
			print STDERR "parser: couldn't open file $file\n";
			next;
    	}
	
		while ($document =~ /<programme/go) {
			$max++;
		}

		debug("programme count is $max");

		if ($max == 0) {
			debug("programme count is 0, not trying to parse.");
			print "1/1\n" if ($slave);
		} else {
			XMLTV::parse_callback($document, \&_encoding_cb, \&_credits_cb,
				\&_channel_cb, \&_programme_cb);
			
			persist_programs() unless defined ($programs_persisted);
			persist_channels() unless defined ($channels_persisted);
		}
		my $stop = time;
		debug("Parsing $file took " . ($stop - $start) . " seconds.");
	}
	my $stop = time;

	debug("Parsing xmltv file(s) took " . ($stop - $all_start) ." seconds.");
}

sub persist_programs {
	$programs_persisted = 1;
	my %dates = ();

	foreach my $date (keys %{$tree}) {
		mkdir ($datadir . "/" . $date);
	
		foreach my $channel (keys %{$tree->{$date}}) {
			if (!open (WRITE, ">:utf8", $datadir . "/" . $date . "/" . $channel . ".dat")) {
				debug("Cannot open " . $datadir . "/" . $date . "/" . $channel . ".dat");
				next;
			}
			store_fd $tree->{$date}->{$channel}, *WRITE;
			close WRITE;
		}
	}
}

sub persist_channels {
	$channels_persisted = 1;

	my $existing_channels;

	if (open (READ, "<:utf8", $confdir . "/channels.dat")) {
		$existing_channels = fd_retrieve *READ;
		close READ;
	} else {
		debug("no existing channels");

		# Legacy format for the channels, read it in and get rid of it.
		if (-e $confdir . "/channels.conf") {
			debug("reading legacy channels");
			if (open (READ, "<:utf8", $confdir."/channels.conf")) {
				my $order = 0;
				while (<READ>) {
					chomp;
					my @tmp = split (/:/, $_);
					if (scalar @tmp == 3) {
						my $c = {id       => $tmp[1],
							name     => $tmp[2],
							order    => $order++,
							visible  => $tmp[0]};
						$existing_channels->{$c->{id}} = $c;
					}
				}
				close READ;
			}
		}
	}

	# Append all new channels to the channel configuration

	foreach my $c (values %$channels) {
		unless (defined ($existing_channels->{$c->{id}})) {
			$existing_channels->{$c->{id}} = $c;
			debug("New channel detected: " . $c->{id});
		}
	}

	if (open (WRITE, ">:utf8", $confdir . "/channels.dat")) {
		store_fd $existing_channels, *WRITE;
		close WRITE;

		# Remove now the legacy channel configuration, if it exists
		if (-e $confdir . "/channels.conf") {
			unlink $confdir . "/channels.conf";
		}
	} else {
		debug("Couldn't save channels");
	}
}

1;
