package GShowTV::Main;

use strict;
use warnings;
use Glib qw/TRUE FALSE/;
use Gtk2;

use Data::Dumper;

use GShowTV::UI::Application;
use GShowTV::Strings qw/time2str/;
use GShowTV::Debug qw/debug/;
use GShowTV::Config;
use GShowTV::Constants;
use GShowTV::Data::Loader;
use GShowTV::Data::Reminderer;
use GShowTV::Data::PVR;
use GShowTV::Favourite;

use GShowTV::Data::FavouriteTask;
use GShowTV::Data::FindTask;
use GShowTV::Data::NotifyTask;

use Locale::TextDomain qw/gshowtv/;

# This is the main handler of the work. It instantiates our Application window and handles the events received

# TODO Should date transition be handled somehow or is it ok to show past dates?

sub new {
	my $self = bless {};
	
	$self->{application} = GShowTV::UI::Application->new();
	$self->{config}      = GShowTV::Config->new();

	$self->{config}->read_config();
	$self->{application}->set_config($self->{config});
	
	$self->{application}->set_current_date(time2str("%Y%m%d", time)); # Set current date to be now

	$self->{application}->signal_connect(quit     => \&do_quit,     $self);
	$self->{application}->signal_connect(record   => \&do_record,   $self);
	$self->{application}->signal_connect(cancel   => \&do_cancel,   $self);
	$self->{application}->signal_connect(watch    => \&do_watch,    $self);
	$self->{application}->signal_connect(nautilus => \&do_nautilus, $self);
	$self->{application}->signal_connect(delete   => \&do_delete,   $self);
	$self->{application}->signal_connect(find     => \&do_find,     $self);

	$self->{application}->signal_connect(config_changed        => \&do_change_config,    $self);
	$self->{application}->signal_connect(channels_changed      => \&do_change_channels,  $self);
	$self->{application}->signal_connect(favourites_changed    => \&do_change_favourites,$self);
	$self->{application}->signal_connect(xmltv_druid_activated => \&do_xmltv_druid,      $self);
	$self->{application}->signal_connect(program_selected      => \&do_program_selected, $self);
	$self->{application}->signal_connect(channel_selected      => \&do_channel_selected, $self);

	$self->{loader} = GShowTV::Data::Loader->new();
	$self->{loader}->set_config($self->{config});
	$self->{loader}->signal_connect(images_ready     => \&do_images_ready,     $self);
	$self->{loader}->signal_connect(xmltv_data_ready => \&do_xmltv_data_ready, $self);
	$self->{loader}->signal_connect(error            => sub {$_[2]->{application}->set_statusbar_text($_[1]);}, $self);
	$self->{loader}->signal_connect(info			 => sub {$_[2]->{application}->set_statusbar_text($_[1]);}, $self);

	$self->{pvr} = GShowTV::Data::PVR->new();
	$self->{pvr}->set_config($self->{config});
	$self->{pvr}->signal_connect(error            => sub {$_[2]->{application}->set_statusbar_text($_[1]);}, $self);
	$self->{pvr}->signal_connect(info             => sub {$_[2]->{application}->set_statusbar_text($_[1]);}, $self);
	$self->{pvr}->signal_connect(pvr_data_ready   => \&do_pvr_data_ready, $self);

	$self->{reminderer} = GShowTV::Data::Reminderer->new();
	$self->{reminderer}->set_config($self->{config});

	my $favourites = GShowTV::Favourite::deserialize();
	$self->{application}->set_favourites($favourites);
	$self->{favourites} = $favourites;	
	$self;
}

sub run {
	my $self = shift;
	
	$self->{loader}->start();
	$self->{reminderer}->start();
	$self->{pvr}->start();
	
	if ( ! defined ($self->{config}->{xmltvfile})) {
		$self->{application}->show_all();
		if (defined ($GShowTV::Constants::xmltv_druid)) {
			debug("Launching xmltv-druid");
			my $file = `$GShowTV::Constants::xmltv_druid`;
			chomp($file);

			debug("Configured XMLTV, data file is: $file");
			
			if ($file eq "") {
				$self->{application}->on_preferences_activate();
			} else {
				my $config = $self->{config};
				$config->{xmltvfile} = $file;

				do_change_config(undef, $config, $self);
			}
		} else {
			$self->{application}->on_preferences_activate();
		}
	}
	
	debug("entering mainloop");
	Gtk2->main();
	debug("came out of mainloop");
}

sub do_quit {
	my ($app, $config, $self) = @_;
	$config->write_config();
	
	debug("Quit signal received from application");

	Gtk2->main_iteration; # Join the main thread
	Gtk2->main_quit;
}

sub do_record {
	my ($app, $p, $self) = @_;
	debug("Record signal received from application");

	if (defined ($self->{pvr_tree}->{$p->{id}})) {
		$self->{application}->set_statusbar_text(__"Cannot set recording, the show is already scheduled for recording.");
		return;
	}

	if ($self->{pvr}->record($p, $self->{channels}->{$p->{channel}})) {
		debug("Recording added to the list");

		$self->{pvr_tree}->{$p->{id}} = $p;
		$self->{application}->set_pvr_tree($self->{pvr_tree});
		$self->{application}->set_statusbar_text(__"Recording scheduled.");
	}
}

sub do_cancel {
	my ($app, $p, $self) = @_;
	debug("Cancel signal received from application");
	
	if ( ! defined ($self->{pvr_tree}->{$p->{id}})) {
		$self->{application}->set_statusbar_text(__"Cannot cancel recording.");
		return;
	}
	
	if ($self->{pvr}->cancel($p, $self->{channels}->{$p->{channel}})) {
		debug("Recording removed from the list");

		delete $self->{pvr_tree}->{$p->{id}};
		$self->{application}->set_pvr_tree($self->{pvr_tree});
		$self->{application}->set_statusbar_text(__"Recording cancelled.");
	}
}

sub do_watch {
	my ($app, $p, $self) = @_;
	debug("Watch signal received from application");
	
	$self->{pvr}->watch($p, $self->{channels}->{$p->{channel}});
}

sub do_nautilus {
	my ($app, $u, $self) = @_;
	debug("Nautilus signal received from application");

	$self->{pvr}->filemanager();
}

sub do_delete {
	my ($app, $p, $self) = @_;
	debug("Delete signal received from application");

	if ( ! defined ($self->{pvr_tree}->{$p->{id}})) {
		$self->{application}->set_statusbar_text(__"Cannot delete recording.");
		return;
	}
	
	if ($self->{pvr}->delete($p, $self->{channels}->{$p->{channel}})) {
		debug("Recording removed from the list");

		delete $self->{pvr_tree}->{$p->{id}};
		$self->{application}->set_pvr_tree($self->{pvr_tree});
		$self->{application}->set_statusbar_text(__"Recording deleted.");
	}
}

sub do_find {
	my ($app, $string, $self) = @_;
	debug("Find signal received from application for $string");
	$self->start_task(GShowTV::Data::FindTask->new(string => $string, list => $self->{application}->{findtreeview}));
}

sub do_xmltv_druid {
	my ($app, $self) = @_;
	debug("Xmltv-druid signal received from application");

	if (defined ($GShowTV::Constants::xmltv_druid)) {
		debug("Launching xmltv-druid");
		my $file = `$GShowTV::Constants::xmltv_druid`;
		chomp($file);

		debug("Configured XMLTV, data file is: $file");
			
		if ($file ne "" && $file ne $self->{config}->{xmltvfile}) {
			my $config = $self->{config};
			$config->{xmltvfile} = $file;

			do_change_config(undef, $config, $self);
		}
	}	
}

sub do_change_channels {
	my ($app, $channels, $self) = @_;
	debug("Channels-changed signal received from application");
	
	GShowTV::Data::IO::write_channels($channels);
}

sub do_change_config {
	my ($app, $config, $self) = @_;
	debug("Config-changed signal received from application $config");
	$self->{config} = $config;
	$self->{loader}->set_config($self->{config});
	$self->{reminderer}->set_config($self->{config});
	$self->{pvr}->set_config($self->{config});
	$config->write_config();
}

sub do_change_favourites {
	my ($loader, $favourites, $self) = @_;
	$self->{favourites} = $favourites;
	debug("Favourites-changed signal received from application");
	$self->start_task(GShowTV::Data::FavouriteTask->new(list => $self->{application}->{favouritestreeview}));
	$self->start_task(GShowTV::Data::NotifyTask->new(reminderer => $self->{reminderer}));
	GShowTV::Favourite::serialize($favourites);

	$self->{application}->set_favourites($favourites);	
}

sub do_xmltv_data_ready {
	my ($loader, $programs, $self) = @_;
	debug("Xmltv data ready signal received from loader");
	
	my $channels = $self->{loader}->get_channels();

	$self->{application}->set_channels($channels);
	$self->{application}->set_program_tree($programs);
	$self->{application}->show_all();
	$self->{program_tree} = $programs;
	$self->{channels}     = $channels;

	$self->{reminderer}->set_channels($channels);
	
	$self->start_task(GShowTV::Data::FavouriteTask->new(list => $self->{application}->{favouritestreeview}));
	$self->start_task(GShowTV::Data::NotifyTask->new(reminderer => $self->{reminderer}));
}

sub do_images_ready {
	my ($loader, $programs, $self) = @_;
	debug("Images ready signal received from loader");
	$self->{loader}->load_channels();
	$self->{application}->set_channels($self->{loader}->get_channels());
	$self->{reminderer}->set_channels($self->{loader}->get_channels());
}

sub do_pvr_data_ready {
	my ($pvr, $programs, $self) = @_;
	debug("PVR data ready signal received from loader");
	
	my $pvr_channels = $self->{pvr}->get_channels();

	foreach my $c (values %$pvr_channels) {
		my $cc = $self->{channels}->{$c->{id}};
		next if (defined ($cc));
		
		$self->{channels}->{$c->{id}} = $c;
	}

	$self->{application}->set_channels($self->{channels}, TRUE);
	$self->{application}->set_pvr_tree($programs);
	$self->{pvr_tree} = $programs;
}

sub do_program_selected {
	my ($self, $p) = @_;
	debug("Program-selected signal received from application for $p->{id}");
}

sub do_channel_selected {
	my ($self, $c) = @_;
	debug("Channel-selected signal received from application for $c->{id}");
}

sub start_task {
	my $self = shift;
	my $task = shift;

	# Check if a task of this type is running
	my $ref = ref $task;
	my $pid = $self->{running_tasks}->{$ref};
	
	if (defined ($pid)) {
		debug("Cancelling old task $pid");
		Glib::Source->remove($pid);
	}

	$task->set_programs($self->{program_tree});
	$task->set_config($self->{config});
	$task->set_favourites($self->{favourites});
	$task->set_channels($self->{channels});
	
	# Add the task and make it remove itself automatically when it's done
	$pid = Glib::Idle->add(sub { 
		my $task = shift; 
		if ($task->run()) {
			TRUE;
		} else {
			debug("Stopping task " . (ref $task) . " with pid " . $task->{pid});
			Glib::Source->remove($task->{pid});
			FALSE;
		}
	}, $task);
	debug("Started a task " . (ref $task) . " with pid $pid");
	$task->{pid} = $pid;
	$self->{running_tasks}->{$ref} = $pid;
}

1;
