#
# This package contains all string handling methods that are widely used within the application.
#
package GShowTV::Strings;

use utf8;

use Exporter 'import';
@EXPORT_OK = qw /time2str str2time is_old_date myglob format_time format_right_notebooktime format_hours/;

use strict;
use warnings;

use Glib qw/TRUE FALSE/;

use Time::Local;
use GShowTV::Constants;
use Locale::TextDomain qw/gshowtv/;

sub time2str {
  # Our own implementation of this method, one less dependency
  my ($format, $time) = @_;

  my @t = localtime $time;

  my $str = $format;

  if ($str =~ /\%Y/) {
    my $Y  = ($t[5] + 1900);
    $str =~ s/\%Y/$Y/g;
  }

  if ($str =~ /\%m/) {
    my $m = ($t[4] < 9 ? ("0" . ($t[4] + 1)) : ($t[4] + 1));
    $str =~ s/\%m/$m/g;
  }

  if ($str =~ /\%d/) {
    my $d   = ($t[3] < 10 ? ("0" . ($t[3])) : ($t[3]));
    $str =~ s/\%d/$d/g;
  }

  if ($str =~ /\%A/) {
    my $A = "" . $GShowTV::Constants::weekdays[$t[6]];
    $str =~ s/\%A/$A/g;
  }

  if ($str =~ /\%B/) {
    my $B = "" . $GShowTV::Constants::months[$t[4]];
    $str =~ s/\%B/$B/g;
  }

  if ($str =~ /\%H/) {
    my $H = ($t[2] < 10 ? ("0" . $t[2]) : $t[2]);
    $str =~ s/\%H/$H/g;
  }

  if ($str =~ /\%M/) {
    my $M = ($t[1] < 10 ? ("0" . $t[1]) : $t[1]);
    $str =~ s/\%M/$M/g;
  }

  if ($str =~/\%S/) {
    my $S = ($t[0] < 10 ? ("0" . $t[0]) : $t[0]);
    $str =~ s/\%S/$S/g;
  }

  die "Unknown timeformatting symbol remains $str." if ($str =~ /\%/);
  $str;
}

sub str2time {
	my ($format, $str) = @_;

	return undef unless defined ($str);

	if ($format eq "%Y%m%d") {
		$str =~/(\d\d\d\d)(\d\d)(\d\d)/;
		timelocal(0, 0, 0, $3, $2 - 1, $1 - 1900);
	} elsif ($format eq "%Y%m%d%H%M") {
		$str =~/(\d\d\d\d)(\d\d)(\d\d)(\d\d)(\d\d)/;
		timelocal(0, $5, $4, $3, $2 - 1, $1 - 1900);
	} elsif ($format eq "%Y%m%d%H%M%S") {
		$str =~/(\d\d\d\d)(\d\d)(\d\d)(\d\d)(\d\d)(\d\d)/;
		timelocal($6, $5, $4, $3, $2 - 1, $1 - 1900);
	} else {
		die "Cannot use other time formats except '%Y%m%d', '%Y%m%d%H%M', '%Y%m%d%H%M%S'.";
	}
}

sub is_old_date {
	my $date = shift;

	return FALSE unless defined $date;

	if (length($date) == 8) {
		$date = $date . "235900";
	} elsif (length($date) == 12) {
		$date = $date . "00";
	}
	
	if (str2time("%Y%m%d%H%M%S", $date) < time) {
		TRUE;
	} else {
		FALSE;
	}
}

sub myglob {
  my $glob = shift;

  my ($res) = glob $glob;

  return $res;
}

sub format_time {
	my $config = shift;
	my $p      = shift;

	my $str;

	if (defined $p->{'start'}) {
		$p->{'start'} =~ /\d{8}(\d\d)(\d\d)\d\d/;
		$str = format_hours($config->{use24htime}, $1,$2);
	} else {
		$str = "??:??";
	}

	if (defined $p->{'stop'}) {
		$p->{'stop'} =~ /\d{8}(\d\d)(\d\d)\d\d/;
		$str .= "-" . format_hours($config->{use24htime}, $1,$2);
	} else {
		$str .= "-??:??";
	}

	$str;
}

sub format_hours {
	my $use24htime = shift;
	my $h = shift;
	my $m = shift;

	$m = "0" . $m if (length($m) < 2);
	
	if ($use24htime) {
		$h . ":" . $m;
	} else {
		if ($h < 12) {
			$h . ":" . $m . __" AM";
		} elsif ($h == 12) {
			$h . ":" . $m . __" PM";
		} else {
			$h = $h - 12;
			($h < 10 ? "0$h" : $h) . ":" . $m . __" PM";
		}
	}
}

sub format_right_notebooktime {
	my $config = shift;
	my $p = shift;
	
	my $str;

	if (defined $p->{'start'}) {
		$p->{'start'} =~ /(\d\d\d\d)(\d\d)(\d\d)(\d\d)(\d\d)\d\d/;
		my $t = str2time("%Y%m%d", $1.$2.$3);
		$str = time2str($config->{dateformat}, $t) . " " . format_hours($config,$4,$5);
	} else {
		$str = "??-??-?? ??:??";
	}

	if (defined $p->{'stop'}) {
		$p->{'stop'} =~ /(\d\d\d\d)(\d\d)(\d\d)(\d\d)(\d\d)\d\d/;
		$str = $str . "-" . format_hours($config, $4, $5);
	} else {
		$str = $str . "-??:??";
	}

	$str . " ";
}
	
1;
