###############################################################################
# The Grid display. Not perfect by any means, but more-or-less does the job.
###############################################################################

package GShowTV::UI::GridDisplay;
use Glib qw(TRUE FALSE);
use Gtk2;

use strict;
use warnings;

use GShowTV::Debug qw/debug/;
use GShowTV::Strings qw/time2str str2time format_hours/;

use Locale::TextDomain qw/gshowtv/;

use Glib::Object::Subclass
	Gtk2::HBox::, # Parent class
	signals => {
		program_selected => {
			method      => 'default_signal_handler',
			flags       => [qw/run-first/],
			return_type => undef, # void return
			param_types => [qw/Glib::Scalar/], # instance and data are automatic
		},
	},
	properties => [
		Glib::ParamSpec->scalar ('channels',
			'Channels',
			'Ref to channels',
			[qw/readable writable/]),
		Glib::ParamSpec->scalar ('programs',
			'Programs',
			'Ref to programs',
			[qw/readable writable/]),
		Glib::ParamSpec->boolean('use24htime',
			'Use 24h Time',
			'Use 24h formatted time',
			FALSE,
			[qw/readable writable/]),
		Glib::ParamSpec->string ('date',
			'Date',
			'The date of this grid, in YYYYMMDD format',
			undef,
			[qw/readable writable/]),
		Glib::ParamSpec->string ('time',
			'Time',
			'The time of this grid, in HHMM format',
			undef,
			[qw/readable writable/]),
	];

use constant {
	# The colors are defined in Gnome HIG guide,
	# and the recommendation is to use the same palette

	COLOR_OLD => Gtk2::Gdk::Color->new(234 * 257, 232 * 257, 227 * 257), # Basic 3D Hilight
	COLOR_NEW => Gtk2::Gdk::Color->new(255 * 257, 255 * 257, 255 * 257), # White
	COLOR_NOW => Gtk2::Gdk::Color->new(197 * 257, 210 * 257, 200 * 257), # Green Hilight
	COLOR_MARKER => Gtk2::Gdk::Color->new(223 *257, 66 * 257, 30 * 257), # Accent Red

	# Sets the size divider, affects how big the 30min span is
	SIZE_FACTOR => 2.5,
};

sub INIT_INSTANCE {
	my $self = shift;

	debug("INIT_INSTANCE");

	$self->set_homogeneous(FALSE);
	$self->set_spacing(0);

	my $rightvbox     = Gtk2::VBox->new(FALSE, 0);
	my $leftvbox      = Gtk2::VBox->new(FALSE, 0);

	my $hourscroll    = Gtk2::ScrolledWindow->new();
	my $hourfixed     = Gtk2::Fixed->new();

	my $programscroll = Gtk2::ScrolledWindow->new();
	my $programfixed  = Gtk2::Fixed->new();

	my $channelscroll = Gtk2::ScrolledWindow->new();
	my $channelfixed  = Gtk2::Fixed->new();

	$hourscroll->add_with_viewport($hourfixed);
	$programscroll->add_with_viewport($programfixed);
	$channelscroll->add_with_viewport($channelfixed);

	$programscroll->set_policy('automatic', 'automatic');
	$hourscroll->set_policy(   'never',     'never');
	$channelscroll->set_policy('never',     'never');

	$rightvbox->pack_start($hourscroll,  FALSE, FALSE, 0);
	$rightvbox->pack_end($programscroll, TRUE,  TRUE, 0);

	# Top left corner is empty, but put this there to make it's size ok
	my $label = Gtk2::Label->new(" ");

	$leftvbox->pack_start($label,       FALSE, FALSE, 0);
	$leftvbox->pack_end($channelscroll, TRUE,  TRUE, 0);

	$self->pack_start($leftvbox, FALSE, FALSE, 0);
	$self->pack_end($rightvbox,  TRUE,  TRUE, 0);

	# Marker for the current time
	my $nowmarker = Gtk2::EventBox->new();
	$nowmarker->modify_bg('normal', COLOR_MARKER);

	$self->{nowmarker} = $nowmarker;

	# Scroll the hour window along with the program window
	$programscroll->get_hadjustment()->signal_connect("value_changed" => sub {
		# Gtk2::Adjustment
		my ($adj) = @_;
		$hourscroll->set_hadjustment($adj);
	});
	# Scroll the channel window along with the program window
	$programscroll->get_vadjustment()->signal_connect("value_changed" => sub {
		# Gtk2::Adjustment
		my ($adj) = @_;
		$channelscroll->set_vadjustment($adj);
	});

	$self->{hourfixed}       = $hourfixed;
	$self->{programfixed}    = $programfixed;
	$self->{channelfixed}    = $channelfixed;
	$self->{programscroll}   = $programscroll;
	$self->{tooltips}        = Gtk2::Tooltips->new();

	# Set some minimum size requests to prevent the widget from
	# requesting way too big size

	my $dummylabel = Gtk2::Label->new("X");

	$programscroll->set_size_request(1, 1);
	$hourscroll->set_size_request(1, $dummylabel->size_request()->height() + 2);
	$channelscroll->set_size_request(-1, 1);

	# Set up an event handler for the expose event, as it seems to be _the_ only
	# way how to scroll the scrolled window to NOW when clicking on today.

	my $id = $self->signal_connect(expose_event => \&expose_event);
	$self->{expose_event_handler_id} = $id;

	# Setup an autoupdater of the current time

	my $timeoutid = Glib::Timeout->add(1 * 60 * 1000, sub {
		my $self = shift;
		$self->_set_time(time2str("%Y%m%d%H%M", time));
		1;
	}, $self);
	$self->{timeoutid} = $timeoutid;
}

sub SET_PROPERTY {
	my ($self, $pspec, $newval) = @_;

	debug("SET_PROPERTY: " . $pspec->get_name . " = " . $newval);

	$self->{$pspec->get_name} = $newval;

	# If we have all the components set, then we can actually
	# put the missing components to their places

	if (defined ($self->{time}) &&
		defined ($self->{date}) &&
		defined ($self->{channels}) &&
		defined ($self->{programs}) &&
		defined ($self->{use24htime})) {
		$self->_render();
	}
}

sub FINALIZE_INSTANCE {
	my $self = shift;

	my $tid = delete $self->{timeoutid};
	debug ("FINALIZE_INSTANCE: removing timeout $tid");

	if (defined ($tid)) {
		Glib::Source->remove($tid);
	}
}

sub _escape {
	my $t = shift;
	return "" unless defined ($t);
	$t =~ s/&/&amp;/g;
	$t =~ s/</&lt;/g;
	$t =~ s/>/&gt;/g;
	$t;
}

sub _render {
	my $self = shift;

	$self->calculate_channel_widget_size();
	$self->calculate_text_element_size();
	
	my ($min, $max) = $self->calculate_timespan();

	my $date = $self->{date};
	my $time = $self->{time};
	
	# Render channel titles or images

	my $maxwidth = $self->{channelwidth} + 4;

	my $row = 0;
	foreach my $chan (sort { $a->{order} <=> $b->{order} } values %{$self->{channels}}) {
		next unless ($chan->{visible});

		my $widget = $self->get_channel_widget($chan);
		$self->{channelfixed}->put($widget, 2 + ($self->{channelwidth} - $widget->size_request()->width()) / 2, 2 + $row * ($self->{unitheight} + 4));

		if ($maxwidth < $widget->size_request()->width()) {
			$maxwidth = $widget->size_request()->width() + 4;
		}
		$row++;
	}

	$self->{channelfixed}->set_size_request($maxwidth);

	# Render the program data

	$row = 0;
	foreach my $chan (sort { $a->{order} <=> $b->{order} } values %{$self->{channels}}) {
		next unless ($chan->{visible});
		my $thereareprograms = FALSE;
#    print "Adding $chan->{name}..\n";
		foreach my $p (values %{$self->{programs}->{$date}->{$chan->{id}}}) {
			$thereareprograms = TRUE;
			$p->{start} =~ /^\d{8}(\d\d)(\d\d)/;
			my $sh = $1;
			my $sm = $2;
			next unless defined $p->{stop};
			$p->{stop} =~ /^\d{8}(\d\d)(\d\d)/;
			my $eh = $1;
			my $em = $2;

			my $start = $sh * 60 + $sm;
			my $end   = $eh * 60 + $em;

			$end = $end + 24 * 60 if ($end < $start);

			my $delta = $end - $start;

#      print "$p->{title} $sh:$sm-$eh:$em = $delta\t$start-$end\n";

			my $ebox  = Gtk2::EventBox->new();
			my $vbox  = Gtk2::VBox->new(0,5);
			my $title = Gtk2::Label->new();
			my $desc  = Gtk2::Label->new();

			$ebox->add($vbox);
			$vbox->pack_start($title, FALSE, FALSE, 0);
			$vbox->pack_end($desc, TRUE, TRUE, 0);
			
			$title->set_markup("<b>" . _escape($p->{title}) . "</b>");
			$title->set_justify('left');
			$title->set_alignment(0,0);

			# Add some extra info
			$desc->set_markup(_escape($p->{desc}));
			$desc->set_justify('left');
			$desc->set_alignment(0,0);
			$desc->set_line_wrap(TRUE);
			$desc->set_size_request($delta == 0 ? 0 : $delta * $self->{unitwidth} - 4);

			$vbox->set_size_request($delta == 0 ? 0 : $delta * $self->{unitwidth} - 4, $self->{unitheight});
			$vbox->{"user-data"} = $p;

			$ebox->signal_connect("button-press-event" => sub {
				my $self  = shift;
				my $event = shift;
				my $grid  = shift;
				my $p     = $self->get_child()->{"user-data"};

				$self->set_state('selected');
				my $e = $grid->{"selected-ebox"};
				if (defined ($e) && $e != $self) {
					$e->set_state('normal');
				}
				$grid->{"selected-ebox"}    = $self;

				$grid->signal_emit("program_selected", $p);
				$grid->signal_emit("button-press-event", $event);
				1;
			}, $self);

			my $color = $self->_get_color($p);

			$ebox->modify_bg('normal', $color);
			$ebox->{"user-data"} = $color;

			my $tooltip = format_hours($self->{use24htime}, $sh, $sm) . "-" . 
				format_hours($self->{use24htime}, $eh, $em) . "\n" .
				$chan->{name} . "\n" . $p->{title};

			if (defined $p->{desc}) {
				$tooltip = $tooltip . "\n" . $p->{desc};
			}

			$self->{tooltips}->set_tip($ebox, $tooltip, undef);
			$self->{programfixed}->put($ebox, 2 + ($start - $min) * $self->{unitwidth}, 2 + $row * ($self->{unitheight} + 4));
		}

	    # Put a wide spread label on empty rows. Fixes GTK issues with empty lines.

		unless ($thereareprograms) {
			my $label = Gtk2::Label->new(" ");

			$label->set_size_request(($max - $min) * $self->{unitwidth}, $self->{unitheight});
			$self->{programfixed}->put($label, 0, 2 + $row * ($self->{unitheight} + 4));
		}
		$row++;
	}

	# Render hours to the hour list

	for (my $i = 0 ; $i <= $max - $min ; $i = $i + 30) {
		# Round the first occurance to half hour or an hour
		if ($i == 0) {
			$i = 30 - $min % 30;
		}
		my $h = int(($min + $i) / 60);
		my $m = $min + $i - $h * 60;
		my $l = Gtk2::Label->new(format_hours($self->{use24htime}, $h % 24, $m));
		$l->set_justify('left');
		$l->set_alignment(0,0);
		$self->{hourfixed}->put($l, $self->{unitwidth} * $i, 0);
	}

	# Set size for the now marker
	$self->{nowmarker}->set_size_request(1, $row * ($self->{unitheight} + 4));

	# Put the nowmarker to it's correct place, if time is our date.

	if ($time =~ /$date/) {
		$time =~ /(\d{8})(\d\d)(\d\d)/;
		my $nowminutes = $2 * 60 + $3;
		$self->{programfixed}->put($self->{nowmarker}, ($nowminutes - $self->{min}) * $self->{unitwidth}, 0);

		debug("Placing nowmarker to $nowminutes");
	}

	$self->show_all();
}

sub calculate_channel_widget_size {
	my $self = shift;
	
	foreach my $chan (values %{$self->{channels}}) {
		next unless ($chan->{visible});
		
		my $w = $self->get_channel_widget($chan);
		my $r = $w->size_request();

		$self->{unitheight}   = $r->height() if (! defined ($self->{unitheight}) || $self->{unitheight} < $r->height());
		$self->{channelwidth} = $r->width()  if (! defined ($self->{channelwidth})  || $self->{channelwidth} < $r->width());
	}

	debug("Calculated image HxW: " . $self->{channelwidth} ."x"  .$self->{unitheight});
}

sub calculate_timespan {
	my $self = shift;
	my ($min, $max);

	# Find out the minimum and maximum

	foreach my $chan (values %{$self->{channels}}) {
		next unless ($chan->{visible});
		foreach my $p (values %{$self->{programs}->{$self->{date}}->{$chan->{id}}}) {
			$p->{start} =~ /^\d{8}(\d\d)(\d\d)/;
			my $sh = $1;
			my $sm = $2;
			next unless defined ($p->{stop});
			$p->{stop} =~ /^\d{8}(\d\d)(\d\d)/;
			my $eh = $1;
			my $em = $2;

			my $start = $sh * 60 + $sm;
			my $end   = $eh * 60 + $em;

			$end = $end + 24 * 60 if ($end < $start);

			$min = $start if (!defined ($min) || $start < $min);
			$max = $end   if (!defined ($max) || $end   > $max);
		}
	}
	$max = 24 * 60 unless (defined ($max));
	$min = 0       unless (defined ($min));

	$self->{min} = $min;
	$self->{max} = $max;

	debug("New min:max = $min:$max");

	($min, $max);	
}

sub calculate_text_element_size {
	my $self   = shift;
	my $markup = "<b>X</b>"; 
	
	debug("Image height is " . $self->{unitheight});
	
	# Find out the pixel size of the font, use that to calculate the size of the view
	my $dummylabel = Gtk2::Label->new("");
	$dummylabel->set_use_markup(TRUE);
	
	for (my $i = 0; $i < 100; $i++) {
		$dummylabel->set_markup($markup);
		my $scale = $dummylabel->size_request();

		if ($scale->height() + 5 >= $self->{unitheight}) {
			debug("Calculated unitheight for $i rows with unitheight " . $self->{unitheight} . " " . $scale->height());
			$self->{unitheight} = $scale->height() + 5;
			$self->{unitwidth} = $scale->width() / SIZE_FACTOR;
			last;
		}
		$markup = $markup . "\nX";
	}
#	exit;
}

sub get_channel_widget {
	my $self = shift;
	my $chan = shift;
	my $widget;
	
	if (-e $chan->{icon} && $chan->{icon} ne $GShowTV::Constants::default_channel_image) { # TODO Clean up default channel image bs
		$widget = Gtk2::Image->new_from_file($chan->{icon});
	} else {
		$widget = Gtk2::Label->new();
		$widget->set_markup("<big><b>" . $chan->{name} . "</b></big>");
		$widget->set_justify('center');
	}
	$widget;
}

sub get_use24htime {
  my $self = shift;
  $self->{use24htime};
}

sub _set_time {
  my $self = shift;
  my $time = shift;

  $time =~ /(\d{8})(\d\d)(\d\d)/;
  $self->{time} = $time;

  return if ($1 ne $self->{date});

  my $nowminutes = $2 * 60 + $3;

  foreach my $child ($self->{programfixed}->get_children()) {
    next unless (UNIVERSAL::isa($child, 'Gtk2::EventBox'));
    my $label = $child->get_child();

    my $p = $label->{"user-data"};

    next unless (defined ($p->{start}));

    my $color = $self->_get_color($p, "UPDATE");

    if (defined ($color)) {
      $child->modify_bg('normal', $color);
    }
  }

  if (defined ($self->{nowmarker}->get_parent())) {
    $self->{programfixed}->move($self->{nowmarker}, ($nowminutes - $self->{min}) * $self->{unitwidth}, 0);
  } else {
    $self->{programfixed}->put($self->{nowmarker}, ($nowminutes - $self->{min}) * $self->{unitwidth}, 0);
  }
}

sub _get_color {
  my $self  = shift;
  my $p     = shift;
  my $mode  = shift;
  my $color = undef;

  # If the grid is from today and the show has already been,
  # it should be old color

  if ($self->{time} =~ /$self->{date}/ &&
      str2time("%Y%m%d%H%M", $self->{time}) > str2time("%Y%m%d%H%M%S", $p->{stop})) {
    return COLOR_OLD;
  }

  # If the show is now on, then give it now color.

  if ($self->{time} =~ /$self->{date}/ &&
      str2time("%Y%m%d%H%M", $self->{time}) > str2time("%Y%m%d%H%M%S", $p->{start}) &&
      str2time("%Y%m%d%H%M", $self->{time}) < str2time("%Y%m%d%H%M%S", $p->{stop})) {
    return COLOR_NOW;
  }

  # If we are just UPDATEing the current and old colors,
  # then return now.

  if (defined ($mode) && $mode eq "UPDATE") {
    return undef;
  }

  return $p->{color} if (defined ($p->{color}));

  # If nothing else, return new color.

  return COLOR_NEW;
}

sub scroll_to_time {
  my $self = shift;

  $self->{time} =~ /(\d{8})(\d\d)(\d\d)/;

  return if ($1 ne $self->{date});

  my $adj   = $self->{programscroll}->get_hadjustment();
  my $upper = $adj->upper();

  $self->{time} =~ /(\d{8})(\d\d)(\d\d)/;

  my $position = $2 * 60 + $3 - $self->{min} - 45; # Minutes back

  $position = 0 if ($position < 0);

  my $fraction = 0;

  if ($self->{max} != $self->{min}) {
    $fraction = $position / ($self->{max} - $self->{min});
  }

  my $value = $upper * $fraction;

  debug("Setting adjustment: Time: $self->{time} Upper: $upper Pos: $position Frac: $fraction Val: $value");
  $adj->set_value($value);
  $self->{programscroll}->set_hadjustment($adj);
}

sub set_favourites {
	my $self = shift;
	my $favs = shift;

	debug("Setting favourites");
	$self->{favourites} = $favs;
  
	LOOP: foreach my $child ($self->{programfixed}->get_children()) {
		next unless (UNIVERSAL::isa($child, 'Gtk2::EventBox'));
		my $label = $child->get_child();

		my $p = $label->{"user-data"};

		next unless (defined ($p->{start}));

		foreach my $f (@{$self->{favourites}}) {
			if ($f->isFavourite($p)) {
				my $color = $f->get_color();
			 	if (defined ($color)) {
					$child->modify_bg('normal', $color);
					next LOOP;
				} 
			}
		}
	}
}

sub expose_event {
  my $self  = shift;
  my $event = shift;

  debug("expose_event: $self $event, removing event handler.");

  if ($self->signal_handler_is_connected($self->{expose_event_handler_id})) {
    $self->signal_handler_disconnect($self->{expose_event_handler_id});
    $self->scroll_to_time();
  }
}

sub default_signal_handler {
  my $self = shift;
  my $data = shift;

  debug("do_program_selected: $self $data");
}

sub unselect_all {
	my $self = shift;
	my $e	 = $self->{"selected-ebox"};

	if (defined ($e)) {
		$e->set_state('normal');
	}
}

1;
