#
# The GridView.
# 
package GShowTV::UI::GridView;
use Glib qw(TRUE FALSE);
use Gtk2;

use strict;
use warnings;

use GShowTV::Debug qw/debug/;
use GShowTV::Strings qw/time2str/;
use GShowTV::UI::GridDisplay;
use Locale::TextDomain qw/gshowtv/;

use Glib::Object::Subclass
	Gtk2::Notebook::, # Parent class
	signals => {
		program_selected => {
			method      => 'do_program_selected',
			flags       => [qw/run-first/],
			return_type => undef, # void return
			param_types => [qw/Glib::Scalar/], # instance and data are automatic
		},
		channel_selected => {
			method      => 'default_signal_handler',
			flags       => [qw/run-first/],
			return_type => undef, # void return
			param_types => [qw/Glib::Scalar/], # instance and data are automatic
		},
	};

sub INIT_INSTANCE {
	my $self = shift;
	$self->set_show_tabs(FALSE);
}

sub set_config {
	my $self   = shift;
	my $config = shift;
	my $oldconfig = $self->{config};
	$self->{config} = $config;	

	if (defined ($oldconfig)) {
		if ($config->{useimages} != $oldconfig->{useimages} ||
			$config->{use24htime} != $oldconfig->{use24htime}) {
			$self->redraw();
		}
	}
}

sub set_favourites {
	my $self = shift;
	$self->{favourites} = shift;
	
	for (my $i = 0; $i < $self->get_n_pages(); $i++) {
		my $child = $self->get_nth_page($i);
		$child->set_favourites($self->{favourites});
	}
}

sub set_current_channel {
	my $self = shift;
	$self->{current_channel} = shift;
}

sub set_current_date {
	my $self              = shift;
	my $old               = $self->{current_date};
	$self->{current_date} = shift;

	if (defined ($old) && $old eq $self->{current_date}) {
		$self->scroll_to_time();
	}
	$self->show_grid();
}

sub set_channels {
	my $self = shift;
	$self->{channels} = shift;
	my $noflush = shift;
	
	$self->redraw() unless ($noflush);
}

sub set_program_tree {
	my $self = shift;
	$self->{program_tree} = shift;
	debug("Set program tree, redrawing..");
	
	$self->redraw();
}

sub get_page_num {
	my $self  = shift;
	my $label = shift;
	
	debug("find out page num for " . $label);
	
	for (my $i = 0; $i < $self->get_n_pages(); $i++) {
		my $child = $self->get_nth_page($i);
		if ($self->get_tab_label_text($child) eq $label) {
			debug("Found index page: $i");
			return $i;
		}
	}
	undef;
}

sub show_grid {
	my $self   = shift;
	my $redraw = shift;

	my $index = $self->get_page_num($self->{current_date});

	# This is bloated because the gridview component cannot be built dynamically, and the current date is always readily used

	if (defined ($self->{channels}) && defined ($self->{program_tree}) && defined ($self->{config}) &&
		(defined ($redraw) || ! defined ($index))) {
		# There is no such page, so create a wholly new gridview
		my $grid = GShowTV::UI::GridDisplay->new(
			channels   => $self->{channels},
			programs   => $self->{program_tree},
			use24htime => $self->{config}->{use24htime},
			date       => $self->{current_date},
			time       => time2str("%Y%m%d%H%M", time));
		$grid->set_favourites($self->{favourites});
		$grid->signal_connect("program_selected" => sub { my ($g, $p, $self) = @_; $self->signal_emit("program_selected", $p); }, $self);
		$grid->signal_connect("button-press-event" => \&popupmenu, $self);
		$index = $self->append_page($grid,$self->{current_date});
		$grid->show_all();

		if ($redraw && $self->get_n_pages() > 1) {
			# All this is to prevent a nasty flicker effect when the programs are reset. 
			# Even this way there is a brief flicker, but its LOTS smaller than without
			debug("redraw forced, removing page 0"); 
			$self->set_current_page($index--);
			$self->remove_page(0);
		}
	}

	if (defined $index) {
		debug("Setting page num: " . $index);
		$self->set_current_page($index);
	} else {
		debug("Delaying construction of a grid view since data not yet available.");
	}

	$self->show_all();
}

sub redraw {
	my $self = shift;

	# Remove all pages

	$self->set_current_page(0);
	
	for (my $i=$self->get_n_pages(); $i > 0; $i--) {
		# Don't remove current date. This eliminates a nasty flicker effect when re-setting programs
		$self->remove_page($i-1) unless ($self->get_tab_label_text($self->get_nth_page($i-1)) eq $self->{current_date}); 
	}

	# And show the current page
	$self->show_grid(TRUE);
}

sub scroll_to_time {
	my $self = shift;
	
	my $index = $self->get_page_num($self->{current_date});
	$self->get_nth_page($index)->scroll_to_time();
}

# Unselect other grids
sub do_program_selected {
	my ($self, $p) = @_;
	$self->unselect_all(TRUE);
}

sub unselect_all {
	my $self = shift;
	my $but  = shift;
	
	for (my $i=0; $i < $self->get_n_pages(); $i++) {
		if (defined ($but) && $i == $self->get_current_page()) {
			next;
		}
		$self->get_nth_page($i)->unselect_all();
	}
}

sub popupmenu {
	my ($grid, $event, $self) = @_;
	
	$self->signal_emit("button-press-event", $event);	
}

1;