###############################################################################
# The List view.
###############################################################################

package GShowTV::UI::ListView;
use Glib qw(TRUE FALSE);
use Gtk2;

use strict;
use warnings;

use GShowTV::Debug qw/debug/;
use GShowTV::Strings qw/time2str is_old_date format_time/;
use GShowTV::Favourite;
use Locale::TextDomain qw/gshowtv/;

use Data::Dumper;

use Glib::Object::Subclass
	Gtk2::HBox::, # Parent class
	signals => {
		program_selected => {
			method      => 'default_signal_handler',
			flags       => [qw/run-first/],
			return_type => undef, # void return
			param_types => [qw/Glib::Scalar/], # instance and data are automatic
		},
		channel_selected => {
			method      => 'default_signal_handler',
			flags       => [qw/run-first/],
			return_type => undef, # void return
			param_types => [qw/Glib::Scalar/], # instance and data are automatic
		},
	},
	properties => [
		Glib::ParamSpec->boolean ('useimages',
			'Use channel images',
			'Should we use channel images or not',
			FALSE,
			[qw/readable writable/]),
		Glib::ParamSpec->scalar ('channels',
			'Channels',
			'Ref to channels',
			[qw/readable writable/]),
		Glib::ParamSpec->scalar ('programs',
			'Programs',
			'Ref to programs',
			[qw/readable writable/]),
		Glib::ParamSpec->scalar ('favourites',
			'Favourites',
			'Ref to favourites',
			[qw/readable writable/]),
		Glib::ParamSpec->boolean('use24htime',
			'Use 24h Time',
			'Use 24h formatted time',
			FALSE,
			[qw/readable writable/]),
		Glib::ParamSpec->string ('date',
			'Date',
			'The date of this grid, in YYYYMMDD format',
			undef,
			[qw/readable writable/]),
		Glib::ParamSpec->string ('time',
			'Time',
			'The time of this grid, in HHMM format',
			undef,
			[qw/readable writable/]),
	];

sub INIT_INSTANCE {
	my $self = shift;
	debug("INIT_INSTANCE");

	$self->set_homogeneous(FALSE);
	$self->set_spacing(0);

	$self->{channellist} = Gtk2::SimpleList->new(__"Channel" => 'text', 'cid' => 'hidden');
	$self->{programlist} = Gtk2::SimpleList->new(__"Time" => 'text', __"Program" => 'text', 'program' => 'hidden', 'color' => 'color');
	$self->{programlist}->set_rules_hint(TRUE);

	my $scroll = Gtk2::ScrolledWindow->new();
	$scroll->add($self->{channellist});
	$scroll->set_policy('never', 'always');
	
	$self->pack_start($scroll, FALSE, TRUE, 0);
	
	$scroll = Gtk2::ScrolledWindow->new();
	$scroll->add($self->{programlist});
	$scroll->set_policy('never', 'automatic');

	$self->pack_start($scroll, TRUE, TRUE,  0);
	
	foreach my $col ($self->{programlist}->get_columns) {
		foreach my $cell ($col->get_cell_renderers) {
			$col->add_attribute ($cell, cell_background_gdk => 3);
		}
	}
	
	$self->{channellist}->signal_connect("cursor-changed" => \&on_cursor_changed, $self);
	$self->{programlist}->signal_connect("cursor-changed" => \&on_cursor_changed, $self);
	$self->{programlist}->signal_connect("button-press-event", \&popupmenu, $self);
}

sub set_config {
	my $self   = shift;
	my $config = shift;
	my $oldconfig = $self->{config};
	$self->{config} = $config;	

	if (!defined ($oldconfig) || $config->{useimages} != $oldconfig->{useimages}) {
		$self->{channellist} = Gtk2::SimpleList->new_from_treeview($self->{channellist},
			__"Channel" => $config->{useimages} ? 'pixbuf' : 'text',
			'cid'		=> 'hidden');
		if (defined ($oldconfig)) {
			$self->set_channels($self->{channels});
		}
	}

	if (defined ($oldconfig) && $config->{use24htime} != $oldconfig->{use24htime}) {
		$self->update_program_list();
	}
}

sub set_favourites {
	my $self = shift;
	$self->{favourites} = shift;
	$self->update_program_list();
}

sub set_current_channel {
	my $self = shift;
	my $old  = $self->{current_channel};
	$self->{current_channel} = shift;
	
	if ($self->{current_channel} ne $old) {
		$self->update_program_list();
	}
}

sub set_current_date {
	my $self         = shift;
	my $old_date     = $self->{current_date};
	$self->{current_date} = shift;

	if (!defined ($old_date) || $old_date ne $self->{current_date}) {
		$self->update_program_list();
	}
}

sub set_channels {
	my $self = shift;
	my $oldchannels = $self->{oldchannels};
	$self->{channels} = shift;
	my $noflush = shift;
	
	return if ($noflush);
	
	@{$self->{channellist}->{data}} = ();
	$self->{channellist}->columns_autosize();

	foreach my $c (sort { $a->{order} <=> $b->{order} } values %{$self->{channels}}) {
		next unless ($c->{'visible'});

		unless ($self->{config}->{useimages}) {
			push @{$self->{channellist}->{data}}, [$c->{name}, $c->{id}];
			next;
		}
		if (-e $c->{icon}) {
			my $pix = Gtk2::Gdk::Pixbuf->new_from_file($c->{icon});
			push @{$self->{channellist}->{data}}, [$pix, $c->{id}];
		} else {
			my $pix = Gtk2::Gdk::Pixbuf->new_from_file($GShowTV::Constants::default_channel_image);
			push @{$self->{channellist}->{data}}, [$pix, $c->{id}];
		}
				
		unless (defined ($self->{current_channel})) {
			debug("current_channel not defined, setting " . $c->{id});
			$self->{current_channel} = $c->{id};
		}
	}
	
	if (defined ($oldchannels)) {
		$self->update_program_list();
	}
}

sub set_program_tree {
	my $self = shift;
	$self->{program_tree} = shift;
	$self->update_program_list();
}

sub update_program_list {
	my $self  = shift;
	my $start = time;

	return unless (defined ($self->{current_date}) && defined ($self->{current_channel}));

	@{$self->{programlist}->{data}} = ();
	$self->{programlist}->columns_autosize();

	foreach my $p (sort { $a->{'start'} cmp $b->{'start'} } values %{$self->{program_tree}->{$self->{current_date}}->{$self->{current_channel}}}) {
		next if (! $self->{config}->{viewpastshows} && is_old_date($p->{'stop'}));

		my $color = undef;
		LOOP: foreach my $f (@{$self->{favourites}}) {
			if ($f->isFavourite($p)) {
				$color = $f->get_color();
				last LOOP;
			}
		}
		push @{$self->{programlist}->{data}}, [format_time($self->{config}, $p), $p->{'title'}, $p->{'id'}, $color];
	}

	my $end = time;

	debug("Took " . ($end - $start) . " time.");
}

sub on_cursor_changed {
	my ($list, $self) = @_;

	my $index = ($list->get_selected_indices())[0];
	
	if ($list == $self->{programlist}) {
		my $pid = $self->{programlist}->{data}[$index][2];
		$self->signal_emit("program_selected", $self->{program_tree}->{$self->{current_date}}->{$self->{current_channel}}->{$pid});
	} else {
		$self->set_current_channel($list->{'data'}[$index][1]);
		$self->signal_emit("channel_selected", $self->{channels}->{$list->{data}[$index][1]});
	}
}

sub unselect_all {
	my $self = shift;
	
	$self->{programlist}->get_selection()->unselect_all();
}

sub popupmenu {
	my ($list, $event, $self) = @_;
	
	$self->signal_emit("button-press-event", $event);	
}

1;
